/*
	HTMLget written by Christopher E. Miller
	This code is public domain.
	You may use it for any purpose.
	This code has no warranties and is provided 'as-is'.
*/


//debug = HTMLGET;

import std.string, std.conv;
import std.socket, std.socketstream;


int main(char[][] args)
{
	if(args.length < 2)
	{
		printf("Usage:\n   htmlget <web-page>\n");
		return 0;
	}
	char[] url = args[1];
	int i;
	
	const char[] PROTO = "http://";
	if(url.length >= PROTO.length)
	{
		if(!icmp(url[0 .. PROTO.length], PROTO))
			url = url[PROTO.length .. url.length];
	}
	
	i = std.string.find(url, '#');
	if(i != -1) // Remove anchor ref.
		url = url[0 .. i];
	
	i = std.string.find(url, '/');
	char[] domain;
	if(i == -1)
	{
		domain = url;
		url = "/";
	}
	else
	{
		domain = url[0 .. i];
		url = url[i .. url.length];
	}
	
	uint port;
	i = std.string.find(domain, ':');
	if(i == -1)
	{
		port = 80; // Default HTTP port.
	}
	else
	{
		port = std.conv.toUshort(domain[i + 1 .. domain.length]);
		domain = domain[0 .. i];
	}
	
	debug(HTMLGET)
		printf("Connecting to " ~ domain ~ " on port " ~ std.string.toString(port) ~ "...\n");
	
	Socket sock = new TcpSocket(new InternetAddress(domain, port));
	SocketStream ss = new SocketStream(sock);
	
	debug(HTMLGET)
		printf("Connected!\nRequesting URL \"" ~ url ~ "\"...\n");
	
	if(port != 80)
		domain = domain ~ ":" ~ std.string.toString(port);
	ss.writeString("GET " ~ url ~ " HTTP/1.1\r\n"
		"Host: " ~ domain ~ "\r\n"
		"\r\n");
	
	while(ss.readLine().length) {} // Skip HTTP header.
	while(!ss.eof())
	{
		printf("%.*s\n", ss.readLine());
	}
	
	return 0;
}

